@@ -56,13 +56,15 @@ from __future__ import division, print_function
 import numpy as np
 import scipy.interpolate as intp
 import scipy.ndimage
-#import warnings
+
+from . import util
 
 __all__ = ["backpropagate_2d", "fourier_map_2d", "sum_2d"]
 _verbose = 1
 
 
 def backpropagate_2d(uSin, angles, res, nm, lD, coords=None,
+                     weight_angles=True,
                      onlyreal=False, padding=True, padval=0,
                      jmc=None, jmm=None, verbose=_verbose):
     u""" 2D backpropagation with the Fourier diffraction theorem
@@ -106,6 +108,12 @@ def backpropagate_2d(uSin, angles, res, nm, lD, coords=None,
         Computes only the output image at these coordinates. This
         keyword is reserved for future versions and is not
         implemented yet.
+    weight_angles : bool, optional
+        If `True` weight each backpropagated projection with a factor
+        proportional to the angular distance between the neighboring
+        projections.
+        
+        .. versionadded:: 0.1.1
     onlyreal : bool, optional
         If `True`, only the real part of the reconstructed image
         will be returned. This saves computation time.
@@ -115,7 +123,7 @@ def backpropagate_2d(uSin, angles, res, nm, lD, coords=None,
         the process for input image sizes that are not powers of 2.
     padval : float
         The value used for padding. This is important for the Rytov
-        approximation, where an approximat zero in the phase might
+        approximation, where an approximate zero in the phase might
         translate to 2πi due to the unwrapping algorithm. In that
         case, this value should be a multiple of 2πi. 
         If `padval` is `None`, then the edge values are used for
@@ -182,7 +190,12 @@ def backpropagate_2d(uSin, angles, res, nm, lD, coords=None,
     # This is not a big problem. We only need to multiply the imaginary
     # part of the scattered wave by -1.
 
-    sinogram = uSin
+    # Perform weighting
+    if weight_angles:
+        weights = util.compute_angle_weights_1d(angles).reshape(-1,1)
+        sinogram = uSin * weights
+    else:
+        sinogram = uSin
 
     # Size of the input data
     ln = sinogram.shape[1]
@@ -198,8 +211,8 @@ def backpropagate_2d(uSin, angles, res, nm, lD, coords=None,
     else:
         pad = 0
 
-    padl = int(np.ceil(pad / 2))
-    padr = pad - padl
+    padl = np.int(np.ceil(pad / 2))
+    padr = np.int(pad - padl)
 
     if padval is None:
         sino = np.pad(sinogram, ((0, 0), (padl, padr)),
@@ -208,10 +221,8 @@ def backpropagate_2d(uSin, angles, res, nm, lD, coords=None,
             print("......Padding with edge values.")
     else:
         sino = np.pad(sinogram, ((0, 0), (padl, padr)),
-                      # mode="constant", constant_values=((padval,padval),
-                      #(padval,padval),(padval,padval)))
                       mode="linear_ramp",
-                      end_values=((padval, padval), (padval, padval)))
+                      end_values=(padval,))
         if verbose > 0:
             print("......Verifying padding value: {}".format(padval))
 
@@ -308,6 +319,7 @@ def backpropagate_2d(uSin, angles, res, nm, lD, coords=None,
 
     projection = projection.reshape(A, 1, lN)  # * filter2
 
+
     # Prepare complex output image
     if onlyreal:
         outarr = np.zeros((ln, ln))
@@ -542,6 +554,7 @@ def fourier_map_2d(uSin, angles, res, nm, lD, semi_coverage=False,
         Fsin = np.vstack((Fsin, np.conj(Fsin)))
         ang = np.vstack((ang, ang + np.pi))
 
+
     if jmc is not None:
         jmc.value += 1
 
@@ -795,7 +808,7 @@ def sum_2d(uSin, angles, res, nm, lD, coords=None,
     if jmc is not None:
         jmc.value += 1
 
-    for j in xrange(lenf):
+    for j in range(lenf):
         # Get r (We compute f(r) in this for-loop)
         r[0][:] = coords[0, j]  # x
         r[1][:] = coords[1, j]  # y
@@ -849,4 +862,4 @@ def sum_2d(uSin, angles, res, nm, lD, coords=None,
         if jmc is not None:
             jmc.value += 1
 
-    return f[::-1]
+    return f.reshape(lx,lx)
