@@ -72,6 +72,7 @@ import scipy.ndimage
 
 import odtbrain
 
+from . import util
 
 __all__ = ["backpropagate_3d", "fourier_map_3d", "sum_3d",
            "backpropagate_3d_4pi"]
@@ -103,7 +104,7 @@ def _mprotate(ang, lny, pool, order):
     """
     targ_args = list()
 
-    slsize = int(np.floor(lny / _ncores))
+    slsize = np.int(np.floor(lny / _ncores))
 
     for t in range(_ncores):
         ymin = t * slsize
@@ -150,7 +151,7 @@ def _filter2_func(args):
 
 
 def backpropagate_3d(uSin, angles, res, nm, lD, coords=None,
-                     onlyreal=False, 
+                     weight_angles=True, onlyreal=False, 
                      padding=(True, True), padfac=1.75, padval=None,
                      intp_order=2, dtype=_np_float64,
                      num_cores=_ncores, 
@@ -198,6 +199,12 @@ def backpropagate_3d(uSin, angles, res, nm, lD, coords=None,
         Only compute the output image at these coordinates. This
         keyword is reserved for future versions and is not
         implemented yet.
+    weight_angles : bool, optional
+        If `True` weight each backpropagated projection with a factor
+        proportional to the angular distance between the neighboring
+        projections.
+        
+        .. versionadded:: 0.1.1
     onlyreal : bool
         If `True`, only the real part of the reconstructed image
         will be returned. This saves computation time.
@@ -226,7 +233,7 @@ def backpropagate_3d(uSin, angles, res, nm, lD, coords=None,
         padding (see documentation of `numpy.pad`).
     order : int between 0 and 5
         Order of the interpolation for rotation.
-        See `scipy.ndimage.interpolation.rotate` for details.
+        See :func:`scipy.ndimage.interpolation.rotate` for details.
     dtype : dtype object or argument for np.dtype
         The data type that is used for calculations (float or double).
         Defaults to np.float.
@@ -256,7 +263,6 @@ def backpropagate_3d(uSin, angles, res, nm, lD, coords=None,
     odt_to_ri : conversion of the object function :math:`f(\mathbf{r})` 
         to refractive index :math:`n(\mathbf{r})`.
 
-
     """
     A = angles.shape[0]
     # jobmanager
@@ -275,25 +281,18 @@ def backpropagate_3d(uSin, angles, res, nm, lD, coords=None,
     assert uSin.dtype == np.complex128, "uSin dtype must be complex128."
 
     dtype_complex = np.dtype("complex{}".format(
-        2 * int(dtype.name.strip("float"))))
+        2 * np.int(dtype.name.strip("float"))))
 
     # set ctype
     ct_dt_map = {np.dtype(np.float32): ctypes.c_float,
                  np.dtype(np.float64): ctypes.c_double
                  }
 
-    if len(uSin.shape) != 3:
-        raise ValueError("Input data `uSin` must have shape (A,Ny,Nx).")
-    if len(uSin) != A:
-        raise ValueError("`len(angles)` must be  equal to `len(uSin)`.")
-    if len(list(padding)) != 2:
-        raise ValueError("Parameter `padding` must be boolean tuple of" +
-                         " length 2!")
-    if np.array(padding).dtype is not np.dtype(bool):
-        raise ValueError("Parameter `padding` must be boolean tuple.")
-    if coords is not None:
-        raise NotImplementedError("Output coordinates cannot yet" +
-                                  " be set for the 2D backrpopagation algorithm.")
+    assert len(uSin.shape) == 3, "Input data `uSin` must have shape (A,Ny,Nx)."
+    assert len(uSin) == A, "`len(angles)` must be  equal to `len(uSin)`."
+    assert len(list(padding)) == 2, "Parameter `padding` must be boolean tuple of length 2!"
+    assert np.array(padding).dtype is np.dtype(bool), "Parameter `padding` must be boolean tuple."
+    assert coords is None, "Setting coordinates is not yet supported."
 
     # Cut-Off frequency
     # km [1/px]
@@ -313,8 +312,13 @@ def backpropagate_3d(uSin, angles, res, nm, lD, coords=None,
     # This is not a big problem. We only need to multiply the imaginary
     # part of the scattered wave by -1.
 
-    # save memory
-    sinogram = uSin
+    # Perform weighting
+    if weight_angles:
+        weights = util.compute_angle_weights_1d(angles).reshape(-1,1,1)
+        sinogram = weights * uSin
+    else:
+        # save memory
+        sinogram = uSin
 
     # lengths of the input data
     (la, lny, lnx) = sinogram.shape
@@ -342,11 +346,10 @@ def backpropagate_3d(uSin, angles, res, nm, lD, coords=None,
     # Resize image to next power of two for fourier analysis
     # Reduces artifacts
 
-    padyl = int(np.ceil(pady / 2))
-    padyr = pady - padyl
-    padxl = int(np.ceil(padx / 2))
-    padxr = padx - padyl
-
+    padyl = np.int(np.ceil(pady / 2))
+    padyr = np.int(pady - padyl)
+    padxl = np.int(np.ceil(padx / 2))
+    padxr = np.int(padx - padyl)
 
     #TODO: This padding takes up a lot of memory. Move it to a separate
     # for loop or to the main for-loop.
@@ -357,11 +360,8 @@ def backpropagate_3d(uSin, angles, res, nm, lD, coords=None,
             print("......Padding with edge values.")
     else:
         sino = np.pad(sinogram, ((0, 0), (padyl, padyr), (padxl, padxr)),
-                      # mode="constant", constant_values=((padval,padval),
-                      #(padval,padval),(padval,padval)))
                       mode="linear_ramp",
-                      end_values=((padval, padval), (padval, padval),
-                                  (padval, padval)))
+                      end_values=(padval,))
         if verbose > 0:
             print("......Verifying padding value: {}".format(padval))
 
@@ -574,9 +574,9 @@ def backpropagate_3d(uSin, angles, res, nm, lD, coords=None,
             inarr[:] = filter2[p] * projection[i]
             myifftw_plan.execute()
             filtered_proj[p, :, :] = inarr[
-                                           padyl:padyl + lny,
-                                           padxl:padxl + lnx
-                                          ] / (lNx * lNy)
+                                       padyl:padyl + lny,
+                                       padxl:padxl + lnx
+                                      ] / (lNx * lNy)
 
         # resize image to original size
         # The copy is necessary to prevent memory leakage.
@@ -593,8 +593,6 @@ def backpropagate_3d(uSin, angles, res, nm, lD, coords=None,
             filtered_proj_imag = filtered_proj.imag
         else:
             filtered_proj_imag = None
-            # Free memory
-            del filtered_proj
 
         _mprotate(phi0, lny, pool4loop, intp_order)
 
